//////////////////////////////////////////////////////////////////////
//                                                                  //
// StarCraft For CEDEC                                              //
//                                                                  //
// Copyright (c) 2004-2011 Daisuke Shikata                          //
//                                                                  //
//////////////////////////////////////////////////////////////////////
#pragma once

#include <vector>
#include "..\math\IVector2.h"
#include "..\math\IVector2Util.h"
#include "..\IgoTypes.h"
#include "PatternPrimitive.h"
#include "Aspect.h"

typedef std::vector<pattern::PatternPrimitive>::iterator PrimitiveIter;

namespace pattern
{
class PatternBase
{
public:
	class PatternPrimitiveList
	{
	public:
		void set( PatternPrimitive value )
		{
			primitive[count] = value;
			count++;
		}
		void clear()
		{
			count = 0;
		}
		int count;
		PatternPrimitive primitive[300];
	};

	PatternBase()
	{
		for( int i = 0; i < 16; i++ )
		{
			b_primitive_[i].clear();
			w_primitive_[i].clear();
		}
	}

	~PatternBase()
	{
	}

	virtual void setup()
	{
	}

	// パターン検索
	virtual std::vector< PatternPrimitive > matchPoint( int atter, PatternPrimitive* state, int count, Aspect* aspect )
	{
		std::vector< PatternPrimitive > ret;
		for( int i = 0; i < count; i++ )
		{
			match( atter, state[i], aspect, ret );
		}

		return ret;
	}

	bool match( int attr, PatternPrimitive& point, Aspect* aspect, std::vector< PatternPrimitive >& ret )
	{
		PatternPrimitiveList* primitive = NULL;
		PatternPrimitive *answer;
		if( attr == STONE_BLACK  )
		{
			primitive = b_primitive_;
			answer    = b_answer_;
		}
		else if( attr == STONE_WHITE )
		{
			primitive = w_primitive_;
			answer    = w_answer_;
		}
		
		for( int i = 0; i < 16; i++ )
		{	
			bool set = false;
			for( int j = 0; j < primitive[i].count; j++ )
			{
				set = false;
				IVector2 pos = primitive[i].primitive[j].position + point.position;
				if( pos.x >= Aspect::MAX_X_COUNT )break;
				if( pos.y >= Aspect::MAX_Y_COUNT )break;
				if( pos.x < 0 )break;
				if( pos.y < 0 )break;

				if( aspect->attr[ pos.x ][pos.y] != primitive[i].primitive[j].atter )
				{
					break;
				}
				set = true;
			}

			if( set == true )
			{
				IVector2 ans = answer[i].position + point.position;
				if( ans.x < lowLine_ )break;
				if( ans.y < lowLine_ )break;
				if( ans.x > Aspect::MAX_X_COUNT - lowLine_ )break;
				if( ans.y > Aspect::MAX_Y_COUNT - lowLine_ )break;

				ret.push_back( PatternPrimitive(attr, ans.x, ans.y) );
				//return true;
			}
		}

		return false;
	}
	void setLowLine( int value )
	{
		lowLine_ = value;
	}
	void pushBack( PatternPrimitive& primitive )
	{
		IVector2 center;
		for( int i = 0; i < 16; i++ )
		{
			IVector2 pos;
			PatternPrimitive value;
			pos = IVector2Util::ConvertVecByDownSide( primitive.position, center, (IVector2Util::TYPE_SIDE)(i/4) );
			pos = IVector2Util::ConvertMirror(        pos,                center, (IVector2Util::TYPE_MIRROR)(i%4) );
			value.set( primitive.atter, pos.x, pos.y );
			b_primitive_[i].set( value );
			if( value.atter == STONE_WHITE )
			{
				value.atter = STONE_BLACK;
			}
			else if( value.atter == STONE_BLACK )
			{
				value.atter = STONE_WHITE;
			}

			w_primitive_[i].set( value );
		}
	}

	void setAnswer( PatternPrimitive& answer )
	{
		IVector2 center;
		for( int i = 0; i < 16; i++ )
		{
			IVector2 pos;
			PatternPrimitive value;
			pos = IVector2Util::ConvertVecByDownSide( answer.position, center, (IVector2Util::TYPE_SIDE)(i/4) );
			pos = IVector2Util::ConvertMirror(        pos,             center, (IVector2Util::TYPE_MIRROR)(i%4) );
			value.set( answer.atter, pos.x, pos.y );
			b_answer_[i] = value;

			if( value.atter == STONE_WHITE )
			{
				value.atter = STONE_BLACK;
			}
			else if( value.atter == STONE_BLACK )
			{
				value.atter = STONE_WHITE;
			}

			w_answer_[i] = value;
		}
	}
	
private:
	int lowLine_;


	PatternPrimitive                b_answer_[16];
	PatternPrimitiveList b_primitive_[16];
	PatternPrimitive                w_answer_[16];
	PatternPrimitiveList w_primitive_[16];
};

}